#!/usr/bin/env python
'''
libgogo_mini.py - A library for gogo_mini.py
Copyright (C) 2007  nikosapi (nikosapi@gmail.com)

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
'''

import popen2
import time
import re
import os.path

from threading import Thread
from os import mkdir

class record_thread(Thread):

    def __init__(self, duration, prefix, bitrate, sleep_time, \
        record_dir, vdevice = "/dev/video0", adevice = "/dev/dsp1", input_number = "1", \
        mode = "ntsc", width = "640", height = "480", format = "mpeg4"):

        Thread.__init__(self)
        self.duration = duration
        self.sleep_time = sleep_time
        self.vdevice = vdevice
        self.adevice = adevice
        self.input_number = input_number
        self.mode = mode
        self.width = width
        self.height = height
        self.bitrate = bitrate
        self.format = format
        self.record_dir = record_dir + "/"
        self.prefix = prefix
        self.rec_time = "N/A"
        self.current_bitrate = "N/A"
        self.gorecord_process = None
        self.was_killed = False
        self.sleeping = False
        self.sleep_current_sec = 0
        self.sleep_time_sec = 0

    def gorecord(self):

        self.sleep_time_sec = int(float(self.sleep_time) * 60)

        self.sleeping = True
        for sec in xrange(0,self.sleep_time_sec):
            if self.sleeping == False:
                self.was_killed = True
                break
            else:
                self.sleep_current_sec = sec
                time.sleep(1)
        self.sleeping = False

        timestamp = time.strftime("_%Y-%b-%d_%H%M", time.localtime())
        filename = (self.record_dir + self.prefix + timestamp + ".avi").replace(" ","\ ")
        duration = str(int(float(self.duration) * 60))

        if not self.was_killed:
            gorecord_command = "gorecord -duration " + duration + \
            " -vdevice " + self.vdevice + " -adevice " + self.adevice + " -input " + self.input_number + \
            " -mode " + self.mode + " -width " + self.width + " -height " + self.height + " -bitrate " + \
            self.bitrate + " -format " + self.format + " " + filename

            self.gorecord_process = popen2.Popen3(gorecord_command, True)
            self.gorecord_pid = self.gorecord_process.pid
            gorecord_stderr = self.gorecord_process.childerr

            go_time = "0"	
            while self.gorecord_process.poll() == -1:
                gorecord_output = gorecord_stderr.readline(80)
                gorecord_output = gorecord_output.split()
                i = 0
                for line in gorecord_output:
                    # First find gorecord time output
                    if line.find("Frames:") != -1 and i > 0:
                        go_time = gorecord_output[(i - 1)]
                    if re.match('^..:..\...$',go_time):
                        self.rec_time =  go_time[:-3]
                    # Then find current bitrate
                    if line.find("Video:") != -1 and i < (len(gorecord_output) - 1) and re.match('^....kbps$',gorecord_output[(i + 1)]):
                        self.current_bitrate = gorecord_output[(i + 1)]
                    i = i + 1

    def start_record(self):
        self.thread = Thread(target=self.gorecord)
        self.thread.start()

    def stop_record(self):
        time.sleep(0.5)
        if self.thread.isAlive():
            self.sleeping = False
            try:
                popen2.Popen3("kill " + str(self.gorecord_pid))
            except:
                pass
            self.was_killed = True
            return True
        else:
            self.was_killed = False
            return False

class gogo_settings:

    def __init__(self):
        self.config_dir = os.path.expanduser("~/.gogo-mini")
        self.config_file = self.config_dir + "/settings"
        self.default_save_dir = os.path.expanduser("~/gogo-recordings")
        self.record_dir = self.default_save_dir

        if not os.path.exists(self.config_dir):
            mkdir(self.config_dir)

    def read_settings(self, make_dir = True):
        if os.path.isfile(self.config_file):
            config_file_stream = file(self.config_file)
            self.record_dir = config_file_stream.readline().split("\n")[0]
            config_file_stream.close()
        else:
            if not os.path.exists(self.default_save_dir) and make_dir:
                mkdir(self.default_save_dir)

    def write_settings(self, record_dir):
        config_file_w = file(self.config_file, 'w')
        config_file_w.write(record_dir)
        config_file_w.close()


